#
# Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
#                         University Research and Technology
#                         Corporation.  All rights reserved.
# Copyright (c) 2004-2005 The University of Tennessee and The University
#                         of Tennessee Research Foundation.  All rights
#                         reserved.
# Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
#                         University of Stuttgart.  All rights reserved.
# Copyright (c) 2004-2005 The Regents of the University of California.
#                         All rights reserved.
# Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
# $COPYRIGHT$
# 
# Additional copyrights may follow
# 
# $HEADER$
#
############################################################################
#
# Copyright (c) 2003, The Regents of the University of California, through
# Lawrence Berkeley National Laboratory (subject to receipt of any
# required approvals from the U.S. Dept. of Energy).  All rights reserved.
#
# Initially written by:
#       Greg Kurtzer, <gmkurtzer@lbl.gov>
#
############################################################################
%define version 4.1.6
%define realversion %{version}
#define realversion v3.0.x-201710270304-8779384
%define numversion %(echo %{version} | tr -d . | tr -d [a-z])
%define zeronumversion %( echo %{version} | awk -F. '{printf("%02d%02d%02d",$1,$2,$3)}' )

# Turn off hardened stuff
%undefine _hardened_build
%undefine _annotated_build
#############################################################################
#
# Configuration Options
#
#############################################################################

%bcond_without slurm
%bcond_with blcr
%bcond_without modulefile
%bcond_with debug
%bcond_without ucx
%bcond_without mellanox
%bcond_without gpfs
%bcond_with i8
%bcond_with psm
%if 0%{?rhel} < 8
# hwloc is too old in rhel6 and 7
%bcond_with hwloc
%else
%bcond_without hwloc
%endif
# CUDA?
%bcond_with cuda

# debug?
%if %{with debug}
%define extradebug -debug
%define extradebugmodule -debug
%define debug_options --enable-debug --enable-memchecker --with-valgrind
%else
%define extradebug %{nil}
%define extradebugmodule %{nil}
%define debug_options %{nil}
%endif

# compiler for which we are doing this
%{!?compiler: %global compiler gcc}
%define compilershort %( echo %{compiler} | tr -d 0-9 )
# Treat gcc-toolset-10 and similar differently

# cuda?
%if %{with cuda}
%{!?cudaversion: %global cudaversion 11.0}
%define cudanversion %(echo %{cudaversion} | tr -d .)
%define extracuda -cuda%{cudanversion}
%define extracudamodule /cuda-%{cudaversion}
%if "%{compilershort}" != "nvhpc"
%define cudaload module load cudatoolkit/%{cudaversion}
%define cudalocation %{nil}
%else
%define cudaload %{nil}
%endif
# sadly cuda rpms don't provide this so exclude it
%if 0%{?fedora} || 0%{?rhel} >= 7
%global         __requires_exclude ^libcudart\\.so.*$
%else
%{?filter_setup:
%filter_from_requires /^libcudart\.so/d;
%filter_setup
}
%endif
%else
%define extracuda %{nil}
%define extracudamodule %{nil}
%define cudaload %{nil}
%define cudalocation %{nil}
%endif

# default to long integers?
%if %{with i8}
%define extrai8 -i8
%define extrai8module -i8
%define i8intelflags -i8
%define i8gccflags -fdefault-integer-8
%else
%define extrai8 %{nil}
%define extrai8module %{nil}
%define i8intelflags %{nil}
%define i8gccflags %{nil}
%endif

%define extra %{extrai8}%{extradebug}%{extracuda}
%define extramodule %{extrai8module}%{extradebugmodule}%{extracudamodule}

# type: string (root path to install modulefiles)
%{!?modulefile_path: %global modulefile_path /usr/local/share/Modules/modulefiles/}
# type: string (subdir to install modulefile)
%{!?modulefile_subdir: %global modulefile_subdir openmpi%{extramodule}}
# type: string (name of modulefile)
%{!?modulefile_name: %global modulefile_name %{version}}
# The name of the modules RPM.  Can vary from system to system.
# type: string (name of modules RPM)
%{!?modules_rpm_name: %global modules_rpm_name environment-modules}
# 32 or 64?

# Should we build a debuginfo RPM or not?
# NO because we will not strip it anymore
%define debug_package %{nil}
%define __os_install_post /usr/lib/rpm/brp-compress

%{!?configure_options: %global configure_options %{nil}}

# now, depending on the compiler we need different things - place them all here for eacy access/changing
#
# GCC Compiler
%if "%{compilershort}" == "gcc"
%define compilerruntimesection BuildRequires: gcc-gfortran gcc-c++
%define compilerdevelsection Requires: gcc-gfortran gcc-c++
%define compilerdocsection #nothing here
# do nothing special for gcc for build prep
%define compilerbuildprep %{nil}
# we need a special ldflags
%define compilerldflags -Wl,-z,noexecstack
# this is for configuring modules
%define compilermodulename gcc
%define compilerloadmodule %{nil}
%define cflags %( echo %{optflags} | sed -e 's|-Werror=format-security||' -e 's|-Wp,-D_FORTIFY_SOURCE=2||' -e 's|-fstack-protector-strong||' -e 's|-fstack-clash-protection||' -e 's|-fcf-protection||'  )
%define fcflags %{cflags} %{i8gccflags}
%define prereq %{nil}
%endif
#

# AOCC Compiler
%if "%{compilershort}" == "aocc"
%define compilerruntimesection BuildRequires: %{compiler}-compiler-devel \
Requires: %{compiler}-compiler
%define compilerdevelsection Requires: %{compiler}-compiler-devel
%define compilerdocsection #nothing here
# do nothing special for gcc for build prep
%define compilerbuildprep export CC=clang CXX=clang++ FC=flang ; module load aocc
# we need a special ldflags
%define compilerldflags -Wl,-z,noexecstack
# this is for configuring modules
%define compilermodulename aocc-%( echo %{compiler} | cut -b5 ).%( echo %{compiler} | cut -b6 ).%( echo %{compiler} | cut -b7 )
%define compilerloadmodule module load aocc
%define cflags -O2 -fPIC
%define cxxflags %{cflags}
%define fcflags  %{cflags}
%define prereq prereq aocc
%endif
#

# gcc-toolset-X
%if "%{compilershort}" == "gcc-toolset-"
%define compilerversion %( echo %{compiler} | cut -d- -f3  )
%define compilerruntimesection BuildRequires: %{compiler}-module-devel
%define compilerdevelsection Requires: %{compiler}-module-devel
%define compilerdocsection #nothing here
# do nothing special for gcc for build prep
%define compilerbuildprep module load gcc-toolset
# we need a special ldflags
%define compilerldflags -Wl,-z,noexecstack
# this is for configuring modules
%define compilermodulename %{compiler}
%define compilerloadmodule %{nil}
%define cflags %( echo %{optflags} | sed -e 's|-Werror=format-security||' -e 's|-Wp,-D_FORTIFY_SOURCE=2||' -e 's|-fstack-protector-strong||' -e 's|-fstack-clash-protection||' -e 's|-fcf-protection||'  )
%define fcflags %{cflags} %{i8gccflags}
%define prereq prereq gcc-toolset/%{compilerversion}
%endif
#

# Devtoolset gcc compiler
%if "%{compilershort}" == "devtoolset-"
%define compilershortversion %( echo %{compiler} | sed -e 's|devtoolset\([0-9]\).*|devtoolset-\1|' )
%define compilerdevelsection Requires: %{compilershortversion}-build %{compilershortversion}-toolchain
%define compilerruntimesection Build%{compilerdevelsection}
%define compilerdocsection #nothing here
%define compilerbuildprep source /opt/rh/%{compiler}/enable
# we need a special ldflags
%define compilerldflags -Wl,-z,noexecstack
# this is for configuring modules
%define compilermodulename %{compilershortversion}
%define compilerloadmodule %{nil}
# breaks the build so turn off
%undefine _annotated_build
%endif

# Intel 10.0 or higher compiler
%if "%{compilershort}" == "intel" || "%{compilershort}" == "oneapi" || "%{compilershort}" == "intel-classic"
# check if this is oneapi/new version scheme, i.e. intel20211
%if %( echo %{compiler} | wc -c ) > 10
%define intelmajor %( echo %{compiler} | tr -d [a-z-] | cut -b1-4 )
%if %( echo %{compiler} | tr -d [a-z-] | cut -b5- ) > 1
%define intelminor %( echo %{compiler} | tr -d [a-z-] | cut -b5 ).%( echo %{compiler} | tr -d [a-z-] | cut -b6 )
%else
%define intelminor %( echo %{compiler} | tr -d [a-z-] | cut -b5 )
%endif
%else
# we will extract versions we need out of above string
%define intelmajor %( echo %{compiler} | cut -b6-7 )
%define intelminor %( echo %{compiler} | cut -b8 )
%endif
%define intelversion %{intelmajor}.%{intelminor}
%define intelversionnum %{intelmajor}%{intelminor}
%{!?intelminrelease: %global intelminrelease 0}
%define compilershortversion %{compilershort}-%{intelmajor}
%define compilerruntimesection BuildRequires: intel-compiler%{intelmajor}-default-modules-devel >= %{intelversion}-%{intelminrelease}\
Requires: intel-compiler%{intelmajor}-default-modules >= %{intelversion}\
Provides: openmpi%{extra}-%{compilershort}-runtime = %{version}-%{intelversionnum}.%{release}
%define compilerdevelsection Provides: openmpi%{extra}-%{compilershort}-devel = %{version}-%{intelversionnum}.%{release}\
Requires: intel-compiler%{intelmajor}-default-modules-devel >= %{intelversion}-%{intelminrelease}
%define compilerdocsection Provides: openmpi%{extra}-%{compilershort}-doc = %{version}-%{intelversionnum}.%{release}
# for intel we need to just specify our compilers
%if "%{compilershort}" == "intel" || "%{compilershort}" == "intel-classic"
%define compilerbuildprep export CC=icc CXX=icpc FC=ifort; . /etc/profile.d/modules.sh; module load %{compilershort}
# this is for configuring modules
%define compilermodulename %{compilershort}-%{intelversion}
%define compilerloadmodule module load %{compilershort}/%{intelmajor}
%define prereq prereq %{compilershort}
%else
%define compilerbuildprep export CC=icx CXX=icpx FC=ifx; . /etc/profile.d/modules.sh; module load intel-oneapi
# this is for configuring modules
%define compilermodulename oneapi-%{intelversion}
%define compilerloadmodule module load intel-oneapi/%{intelmajor}
%define prereq prereq intel-oneapi
%endif
%if %{with debug}
%define cflags -O2 -g -pipe -Wall -diag-disable=10441
%else
%define cflags -O2 -pipe -Wall -diag-disable=10441
%endif
%define cxxflags %{cflags}
%if %{with debug}
%define fcflags -O2 -g %{i8intelflags}
%else
%define fcflags -O2 %{i8intelflags}
%endif
# no special ldflags here
%define compilerldflags %{nil}
%endif
#

#
# PGI compiler
%if "%{compilershort}" == "pgi" || "%{compilershort}" == "nvhpc"
# we will extract versions we need out of above string
%if "%{compilershort}" == "pgi"
%define pgimajor %( echo %{compiler} | cut -b4-5 )
%define pgiminor %( echo %{compiler} | cut -b6 )
%define pgirpm  BuildRequires: pgi-workstation = %{pgiversion}\
Requires: pgi-workstation-libs = %{pgiversion}
%define pgirpmdevel Requires: pgi-workstation = %{pgiversion}
%define compilerbuildprep export CC=pgcc CXX=pgcpp FC=pgfortran; . /etc/profile.d/modules.sh; module load %{compilershort}; export CPP=/usr/bin/cpp
%else
%define pgimajor %( echo %{compiler} | cut -b6-7 )
%define pgiminor %( echo %{compiler} | cut -b8 )
%define pgirpm BuildRequires: pu-nvhpc-%{pgimajor}-%{pgiminor} \
Requires: pu-nvhpc-%{pgimajor}-%{pgiminor}
%define pgirpmdevel Requires: pu-nvhpc-%{pgimajor}-%{pgiminor}
%define compilerbuildprep . /etc/profile.d/modules.sh; module load %{compilershort}; export CPP=/usr/bin/cpp
%if %{with cuda}
%define cudalocation =/opt/nvidia/hpc_sdk/Linux_%{_arch}/%{pgimajor}.%{pgiminor}/cuda/%{cudaversion}
%endif
%endif
%define pgiversionnum %{pgimajor}%{pgiminor}
# for pgi we need to just specify our compilers
%define pgiversion %{pgimajor}.%{pgiminor}
%define compilershortversion %{compilershort}-%{pgimajor}
%define compilerruntimesection %{pgirpm} \
Provides: openmpi%{extra}-%{compilershort}-runtime = %{version}-%{pgiversionnum}.%{release}
%define compilerdevelsection %{pgirpmdevel} \
Provides: openmpi%{extra}-%{compilershort}-devel = %{version}-%{pgiversionnum}.%{release}
%define compilerdocsection Provides: openmpi%{extra}-%{compilershort}-doc = %{version}-%{pgiversionnum}.%{release}
%define cflags -O2 -tp=px -fPIC
%define cxxflags %{cflags}
%define fcflags  %{cflags}
# no special ldflags here
%define compilerldflags %{nil}
# this is for configuring modules
%define compilermodulename %{compilershort}-%{pgiversion}
%define compilerloadmodule module load %{compilershort}/%{pgimajor}
%filter_from_requires /^libomp\.so.*/d
%filter_from_requires /^libomptarget\.so.*/d
%filter_from_requires /^libpgkomp\.so.*/d
%filter_setup
%define prereq prereq %{compilershort}
%endif
#
# Pathscale compiler
%if "%{compilershort}" == "pathscale"
# we will extract versions we need out of above string
%define pathscalemajor %( echo %{compiler} | cut -b10 )
%define pathscaleminor %( echo %{compiler} | cut -b11 )
%define pathscaleversion %{pathscalemajor}.%{pathscaleminor}
%define pathscaleversionnum 0%{pathscalemajor}%{pathscaleminor}
%define compilershortversion pathscale-%{pathscalemajor}
%define compilerruntimesection BuildRequires: pathscale-compilers >= %{pathscaleversion}\
Requires: pathscale-compilers-libs >= %{pathscaleversion}\
Provides: openmpi%{extra}-pathscale-runtime = %{version}-%{pathscaleversionnum}.%{release}
%define compilerdevelsection Requires: pathscale-compilers >= %{pathscaleversion}\
Provides: openmpi%{extra}-pathscale-devel = %{version}-%{pathscaleversionnum}.%{release}
%define compilerdocsection Provides: openmpi%{extra}-pathscale-doc = %{version}-%{pathscaleversionnum}.%{release}
# for intel we need to just specify our compilers
%define compilerbuildprep export CC=pathcc CXX=pathCC FC=pathf90; . /etc/profile.d/modules.sh; module load pathscale
%define cflags -O3
%define cxxflags %{cflags}
%define fcflags  %{cflags}
# no special ldflags here
%define compilerldflags %{nil}
# this is for configuring modules
%define compilermodulename pathscale-%{pathscaleversion}
%define compilerloadmodule module load pathscale/%{pathscaleversion}
%endif


#############################################################################
#
# Configuration Logic
#
#############################################################################

%define _prefixtop /usr/local/openmpi%{extramodule}
%define _prefix %{_prefixtop}/%{version}/%{compiler}
%define _mandir %{_prefix}/man
%define _docdir %{_datadir}/doc
%define _infodir %{_datadir}/info
%global _sysconfdir /etc


# These are the default CSES IB values
%define _with_verbs    --with-verbs=/usr

# The following options is supported and used on 32-bit platforms to turn on IB support.
# It can also be used on 64-bit platforms to specify a different IB directory:
#   --with ib[=<dir>]
#
%if %{?_with_ib:1}%{!?_with_ib:0}
# Set default dir is none specified
%define _ib_dir %(set -- %{_with_ib}; echo $1 | grep -v with | sed 's/=//')
%if %(test "%{_ib_dir}" = "" && echo 1 || echo 0)
%define _with_verbs    --with-verbs=/usr/ofed
%else
%define _with_verbs    --with-verbs=%{_ib_dir}
%endif
%endif

# The following option is supported and is used on 64-bit platforms to turn off IB support:
#   --without ib
#
%if %{?_without_ib:1}%{!?_without_ib:0}
%define _with_verbs    %{nil}
%endif

%if %{with blcr}
%define blcrcompileopts --with-ft --with-blcr=/usr --with-blcr-libdir=/usr/%{_lib}
%else
%define blcrcompileopts %{nil}
%endif

#############################################################################
#
# Preamble Section
#
#############################################################################

Summary: A powerful implementation of MPI
Version: %{version}
Name: %{?_name:%{_name}}%{!?_name:openmpi}%{zeronumversion}%{extra}-%{compiler}
Release: 133%{?dist}%{?with_mellanox:.mellanox}
License: BSD
Group: Development/Libraries
Source: openmpi-%{realversion}.tar.bz2
Patch1: fixpgcc.patch
Packager: %{?_packager:%{_packager}}%{!?_packager:%{_vendor}}
Vendor: %{?_vendorinfo:%{_vendorinfo}}%{!?_vendorinfo:%{_vendor}}
Distribution: %{?_distribution:%{_distribution}}%{!?_distribution:%{_vendor}}
Provides: mpi
BuildRoot: /var/tmp/%{name}-%{version}-%{release}-root
%if %{with modulefile}
Requires: %{modules_rpm_name}
%endif
BuildRequires: rsync flex zlib-devel hostname
%if %{with blcr}
BuildRequires: blcr-devel
%endif
%if %{with slurm}
BuildRequires: slurm-devel
%endif
%if %{with ucx}
BuildRequires: librdmacm-devel
BuildRequires: rdma-core
BuildRequires: ucx-devel binutils-devel libibmad-devel libxml2-devel systemd-devel
%if %{with mellanox}
#if %{with cuda}
# mellanox AND cuda - hcoll malfunctioning, do not install or attempt to use
#define ucxopts --with-ucx=/usr  --without-verbs
#else
%define ucxopts --with-ucx=/usr --with-hcoll=/opt/mellanox/hcoll --without-verbs
BuildRequires: hcoll
#endif
%else
%define ucxopts --with-ucx=/usr
%endif
%else
%define ucxopts %{nil}
%endif
%if %{with cuda} && "%{compilershort}" != "nvhpc"
BuildRequires: cudatoolkit%{cudanversion}-devel
%endif
%if %{with gpfs}
BuildRequires: gpfs.base
%endif
BuildRequires: sqlite-devel
%if %{zeronumversion} < 30000
BuildRequires: plfs-core-devel
%endif
BuildRequires: pmix-devel libevent-devel
Requires: pmix
%ifarch ppc64le
BuildRequires: pmix-pmi-devel
Requires: pmix-pmi
%endif

%description
Open MPI is a project combining technologies and resources from
several other projects (FT-MPI, LA-MPI, LAM/MPI, and PACX-MPI) in
order to build the best MPI library available.

This RPM contains all the tools necessary to compile, link, and run
Open MPI jobs.

#############################################################################
#
# Preamble Section (runtime)
#
#############################################################################

%package runtime
Summary: Tools and plugin modules for running Open MPI jobs with %{compiler} compiler
Group: Development/Libraries
Provides: mpi
Provides: openmpi%{extra}-%{compiler}-runtime = %{version}
%if %{with modulefile}
BuildRequires: %{modules_rpm_name}
BuildRequires: %{modules_rpm_name}-local
%endif
%{compilerruntimesection}
BuildRequires: libtool
BuildRequires: numactl-devel
%if %{with hwloc}
BuildRequires: hwloc-devel
Requires: hwloc
%else
BuildRequires: pciutils-devel
Requires: pciutils-libs
%endif
BuildRequires: papi-devel
Requires: librdmacm
%if %{?_with_verbs:1}%{!?_with_verbs:0}
BuildRequires: libibverbs-devel >= 1.1.3, opensm-devel > 3.3.0, librdmacm-devel
BuildRequires: rdma-core
%if %{with psm}
BuildRequires: libfabric-devel libpsm2-devel
%endif
%if %{with debug}
BuildRequires: valgrind-devel
%endif
%ifarch x86_64
%if "%{?rhel}" > "5" && "%{?rhel}" < "8"
BuildRequires: infinipath-psm-devel
%endif
%endif
BuildRequires: libsysfs-devel
%endif
%if %{with modulefile}
Requires: %{modules_rpm_name}
Requires: %{modules_rpm_name}-local
%endif
%if %{with ucx}
Requires: ucx libibmad libxml2
%if %{with mellanox}
Requires: hcoll
%endif
%endif
%if %{with cuda}
Requires: cudatoolkit%{cudanversion}
%endif

%description runtime
Open MPI is a project combining technologies and resources from several other
projects (FT-MPI, LA-MPI, LAM/MPI, and PACX-MPI) in order to build the best
MPI library available.

This subpackage provides general tools (mpirun, mpiexec, etc.) and the
Module Component Architecture (MCA) base and plugins necessary for
running Open MPI jobs with the %{compiler} compiler.

%package runtime-gpfs
Summary: GPFS ROMIO fs driver for Openmpi
Group: Development/Libraries
Requires: %{name}-runtime = %{version}-%{release}

%description runtime-gpfs
GPFS ROMIO filesystem driver, packaged separately to remove dependency from the base package.

#############################################################################
#
# Preamble Section (devel)
#
#############################################################################

%package devel
Summary: Development tools and header files for Open MPI with %{compiler} compiler
Group: Development/Libraries
Requires: %{name}-runtime
#BuildRequires: libibverbs-devel >= 1.1.3, librdmacm-devel, opensm-devel, rdma-core
Requires: libibverbs-devel, librdmacm-devel, rdma-core, opensm-devel
%if %{with psm}
BuildRequires: libfabric-devel libpsm2-devel
Requires: libfabric-devel, libpsm2-devel
%endif
%if %{with hwloc}
Requires: hwloc-devel
Requires: pciutils-devel
%endif
%ifarch x86_64
%if "%{?rhel}" > "5" && "%{?rhel}" < "8"
BuildRequires:          infinipath-psm-devel
Requires:          infinipath-psm-devel
%endif
%endif
%{compilerdevelsection}
Provides: openmpi%{extra}-%{compiler}-devel = %{version}
Provides: openmpi%{extra}-%{compiler} = %{version}
Requires: numactl-devel libtool
%if %{with slurm}
Requires: slurm-devel
%endif
Obsoletes: %{?_name:%{_name}}%{!?_name:openmpi}010203-%{compiler}-devel

%description devel
Open MPI is a project combining technologies and resources from
several other projects (FT-MPI, LA-MPI, LAM/MPI, and PACX-MPI) in
order to build the best MPI library available.

This subpackage provides the development files for Open MPI, such as
wrapper compilers and header files for MPI development with %{compiler} compiler.

#############################################################################
#
# Preamble Section (docs)
#
#############################################################################

%package docs
Summary: Documentation for Open MPI %{compiler} compiler
Group: Development/Documentation
Requires: %{name}-runtime
%{compilerdocsection}
Provides: openmpi%{extra}-%{compiler}-docs = %{version}
Obsoletes: %{?_name:%{_name}}%{!?_name:openmpi}010203-%{compiler}-docs

%description docs
Open MPI is a project combining technologies and resources from several other
projects (FT-MPI, LA-MPI, LAM/MPI, and PACX-MPI) in order to build the best
MPI library available.

This subpackage provides the documentation for Open MPI with %{compiler} compiler.

#############################################################################
#
# Preparatory Section
#
#############################################################################
%prep
rm -rf $RPM_BUILD_ROOT
%setup -q -n openmpi-%{realversion}
%patch1 -p1
# why the heck would they check for /lib and not /lib64?  Argh:
perl -pi -e 's|pmix_ext_install_dir/lib/|pmix_ext_install_dir/lib64/|' configure
#############################################################################
#
# Build Section
#
#############################################################################

%build
# add to module path possible local directories
export MODULEPATH=$MODULEPATH:/usr/local/share/Modules/modulefiles:/opt/share/Modules/modulefiles
# Kill the stack protection and fortify source stuff...it slows things down
# and openmpi hasn't been audited for it yet
RPM_OPT_FLAGS=`echo $RPM_OPT_FLAGS | sed -e 's/-Wp,-D_FORTIFY_SOURCE=.//' -e 's/-fstack-protector-strong//' -e 's/-fstack-protector//' -e 's/-fstack-clash-protection//'`
%if !%{with debug}
RPM_OPT_FLAGS=`echo $RPM_OPT_FLAGS | sed -e 's/-g//'`
%endif

CFLAGS="%{?cflags:%{cflags}}%{!?cflags:$RPM_OPT_FLAGS}"
CXXFLAGS="%{?cxxflags:%{cxxflags}}%{!?cxxflags:$RPM_OPT_FLAGS}"
FCFLAGS="%{?fcflags:%{fcflags}}%{!?fcflags:$RPM_OPT_FLAGS} -I%{_fmoddir}"

%{compilerbuildprep}

%{cudaload}

export RPM_OPT_FLAGS CFLAGS CXXFLAGS FCFLAGS 

# Build in the library search path to any binaries produced, so
# if the runtime package is installed without the modules package
# it will still be able to find openmpi libraries (like libopal)
%configure	--disable-static --enable-shared --with-sge --enable-mpi_thread_multiple --enable-mpi-cxx \
		--with-cma %{?customopts} %{?with_cuda:--with-cuda%{cudalocation}} \
		%{?without_ucx:%{_with_verbs}} %{blcrcompileopts} %{debug_options} \
		--sysconfdir=%{_sysconfdir}/openmpi%{extracudamodule}/%{version}/%{compiler} \
		%{?with_slurm:--with-slurm --with-pmix=/usr --with-libevent=/usr --with-libevent-libdir=/usr/%{_lib}} %{?with_hwloc:--with-hwloc=/usr} \
		%{ucxopts} LDFLAGS="-Wl,-rpath,%{_prefix}/%{_lib} %{compilerldflags}"

# compensate for icc miscompiling ptmalloc
%if "%{compiler}" == "intel121"
perl -pi -e 's|CC = icc|CC = gcc|' opal/mca/memory/Makefile
%endif
# this is necessary because pgi has trouble with hwloc if we configure with --with-hwloc=/usr
# so we configure hwloc with /usr/local but because it actually needs to access that file we
# fix it here
perl -pi -e 's|/usr/local/include/hwloc|/usr/include/hwloc|' opal/include/opal_config.h config.status

%{__make} %{?mflags}

#############################################################################
#
# Install Section
#
#############################################################################
%install
export MODULEPATH=$MODULEPATH:/usr/local/share/Modules/modulefiles:/opt/share/Modules/modulefiles

# we might need to setup modules before installation
%{compilerbuildprep}

%{cudaload}

%{__make} install DESTDIR=$RPM_BUILD_ROOT %{?mflags_install}

# An attempt to make enviornment happier when installed into non /usr path

%if %{with modulefile}
%{__mkdir_p} $RPM_BUILD_ROOT/%{modulefile_path}/%{modulefile_subdir}/%{compilermodulename}

cat <<EOF >$RPM_BUILD_ROOT/%{modulefile_path}/%{modulefile_subdir}/%{compilermodulename}/%{version}
#%Module

# NOTE: This is an automatically-generated file!  (generated by the
# Open MPI RPM).  Any changes made here will be lost a) if the RPM is
# uninstalled, or b) if the RPM is upgraded or uninstalled.

proc ModulesHelp { } {
   puts stderr "This module adds Open MPI%{extra} %{compilermodulename} %{version} to various paths"
}

module-whatis   "Sets up Open MPI%{extra} %{compilermodulename} %{version} in your environment"

#%{cudaload}
#%{compilerloadmodule}
prepend-path PATH "%{_prefix}/bin"
prepend-path LD_LIBRARY_PATH "%{_libdir}"
prepend-path MANPATH "%{_mandir}"
append-path -d { } LOCAL_LDFLAGS "-L%{_libdir}"
append-path -d { } LOCAL_INCLUDE "-I%{_includedir}"
append-path -d { } LOCAL_CFLAGS "-I%{_includedir}"
append-path -d { } LOCAL_FCFLAGS "-I%{_includedir}"
append-path -d { } LOCAL_CXXFLAGS "-I%{_includedir}"
setenv SLURM_MPI_TYPE pmix_v3
setenv MPIHOME %{_prefix}
setenv MPI_HOME %{_prefix}
setenv OPENMPI_HOME %{_prefix}
# only one mpi version at a time
conflict intel-mpi mvapich mpich openmpi openmpi-cuda
%{prereq}
EOF
%endif
# End of modulefile if

# Build lists of files that are specific to each package that are not
# easily identifiable by a single directory (e.g., the different
# libraries).  In a somewhat lame move, we can't just pipe everything
# together because if the user, for example, did --disable-shared
# --enable-static, the "grep" for .so files will not find anything and
# therefore return a non-zero exit status.  This will cause RPM to
# barf.  So be super lame and dump the egrep through /bin/true -- this
# always gives a 0 exit status.

# Runtime files
find $RPM_BUILD_ROOT%{_prefix} -type f -o -type l | \
   sed -e "s@$RPM_BUILD_ROOT@@" | \
   egrep "lib.*.so|mca.*so" > %{compiler}-runtime.files | /bin/true

# Devel files
find $RPM_BUILD_ROOT%{_prefix} -type f -o -type l | \
   sed -e "s@$RPM_BUILD_ROOT@@" | \
   egrep "lib.*\.a|lib.*\.la|.*\.mod" > %{compiler}-devel.files | /bin/true

mkdir -p $RPM_BUILD_ROOT/%{_docdir}
install -m 644 README INSTALL LICENSE $RPM_BUILD_ROOT/%{_docdir}

# remove config.log files
find $RPM_BUILD_ROOT/%{_prefix} -name config.log -exec rm -f '{}' \;

# configs go here
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/openmpi%{extracudamodule}/%{version}/%{compiler}

#############################################################################
#
# Clean Section
#
#############################################################################
%clean
# Remove installed driver after rpm build finished
rm -rf $RPM_BUILD_DIR/%{_name}-%{_version} 

test "x$RPM_BUILD_ROOT" != "x" && rm -rf $RPM_BUILD_ROOT

#############################################################################
#
# Files Section
#
#############################################################################

#
# Sub-package RPMs
#
# Harder than all-in-one.  We list the directories specifically so
# that if the RPM creates directories when it is installed, we will
# remove them when the RPM is uninstalled.  We also have to use
# specific file lists.
#

%files runtime -f %{compiler}-runtime.files
%defattr(-, root, root, -)
%dir %{_prefixtop}
%dir %{_prefixtop}/%{version}
%dir %{_prefixtop}/%{version}/%{compiler}
%dir %{_prefix}
%if %{with cuda}
%dir %{_sysconfdir}/openmpi
%endif
%dir %{_sysconfdir}/openmpi%{extracudamodule}
%dir %{_sysconfdir}/openmpi%{extracudamodule}/%{version}
%dir %{_sysconfdir}/openmpi%{extracudamodule}/%{version}/%{compiler}
%{_sysconfdir}/openmpi%{extracudamodule}/%{version}/%{compiler}/*
%dir %{_prefix}/bin
%dir %{_prefix}/%{_lib}
%dir %{_prefix}/%{_lib}/openmpi
#%doc README INSTALL LICENSE
%dir %{_prefix}/share
%{_prefix}/share/openmpi
%{_prefix}/bin/mpirun
%{_prefix}/bin/mpiexec
%{_prefix}/bin/ompi_info
%{_prefix}/bin/ompi-server
%{_prefix}/bin/orted
%{_prefix}/bin/orterun
%{_prefix}/bin/orte-clean
%if %{numversion} < 400
%{_prefix}/bin/orte-ps
%{_prefix}/bin/ompi-ps
%{_prefix}/bin/ompi-top
%{_prefix}/bin/orte-top
%endif
%if %{numversion} < 200
%{_prefix}/bin/*otfinfo
%{_prefix}/bin/*compress
%endif
%if %{numversion} < 180
%{_prefix}/bin/orte-iof
%{_prefix}/bin/ompi-iof
%else
%{_prefix}/bin/orte-info
%{_prefix}/bin/orte-server
%if %{with ucx}
%{_prefix}/bin/oshrun
%{_prefix}/bin/oshmem_info
%{_prefix}/bin/shmemrun
%endif
%endif
%if %{numversion} < 200
%{_prefix}/bin/vtrun
%{_prefix}/bin/*otfmerge-mpi
%{_prefix}/bin/*otfprofile-mpi
%endif
%if %{numversion} < 180
%{_prefix}/bin/ompi-probe
%{_prefix}/bin/ompi-profiler
%{_prefix}/bin/orte-bootproxy.sh
%endif
%if %{numversion} > 1100 || %{numversion} >= 200
%if %{zeronumversion} < 30100
%{_prefix}/bin/orte-dvm
%endif
%if %{zeronumversion} < 30000
%{_prefix}/bin/orte-submit
%endif
%endif
%if %{with blcr}
%{_prefix}/bin/opal-checkpoint
%{_prefix}/bin/opal-restart
%{_prefix}/bin/orte-checkpoint
%{_prefix}/bin/orte-restart
%{_prefix}/bin/ompi-checkpoint
%{_prefix}/bin/ompi-restart
%endif
%{_prefix}/bin/ompi-clean
%if %{numversion} >= 200
%if %{zeronumversion} < 30100
%{_prefix}/bin/ompi-dvm
%endif
%if %{zeronumversion} < 30000
%{_prefix}/bin/ompi-submit
%endif
%endif
%if %{with modulefile}
%dir %{modulefile_path}/%{modulefile_subdir}
%dir %{modulefile_path}/%{modulefile_subdir}/%{compilermodulename}
%{modulefile_path}/%{modulefile_subdir}/%{compilermodulename}/%{version}
%endif
%if %{with gpfs}
%exclude %{_prefix}/%{_lib}/openmpi/mca_fs_gpfs.so
%files runtime-gpfs
%{_prefix}/%{_lib}/openmpi/mca_fs_gpfs.so
%endif

%files devel -f %{compiler}-devel.files
%defattr(-, root, root, -)
%{_prefix}/include
%{_prefix}/bin/mpicc
%{_prefix}/bin/mpiCC
%{_prefix}/bin/mpic++
%{_prefix}/bin/mpicxx
%{_prefix}/bin/mpif77
%{_prefix}/bin/mpif90
%{_prefix}/bin/mpifort
%{_prefix}/bin/opal_wrapper
# the following are part of vampirtrace
%if %{numversion} < 200
%{_prefix}/bin/vtcc
%{_prefix}/bin/vtcxx
%{_prefix}/bin/vtf77
%{_prefix}/bin/vtf90
%{_prefix}/bin/vtfilter*
%{_prefix}/bin/vtunify*
%{_prefix}/bin/opari
%{_prefix}/bin/*otfaux
%{_prefix}/bin/*otfconfig
%{_prefix}/bin/*otfmerge
%endif
%if %{numversion} > 160
%if %{numversion} < 200
%{_prefix}/bin/vtCC
%{_prefix}/bin/vtc++
%{_prefix}/bin/*otfprofile
%{_prefix}/bin/otfprint
%{_prefix}/bin/*otfshrink
%{_prefix}/bin/vtfort
%{_prefix}/bin/vtwrapper
%{_prefix}/share/vampirtrace
%{_prefix}/bin/mpi*vt
%if "%{?rhel}" == "6"
%{_prefix}/bin/vtsetup*
%endif
%endif
%if %{numversion} < 180
%{_prefix}/bin/orteCC
%{_prefix}/bin/orte_wrapper_script
%{_prefix}/bin/ortec++
%endif
%{_prefix}/bin/ortecc
%{_prefix}/%{_lib}/pkgconfig
%endif
%if %{with ucx}
%{_prefix}/bin/oshcc
%{_prefix}/bin/oshfort
%{_prefix}/bin/shmemcc
%{_prefix}/bin/shmemfort
%{_prefix}/bin/oshCC
%{_prefix}/bin/oshc++
%{_prefix}/bin/oshcxx
%{_prefix}/bin/shmemCC
%{_prefix}/bin/shmemc++
%{_prefix}/bin/shmemcxx
%endif
%if %{zeronumversion} > 30099
%{_prefix}/bin/aggregate_profile.pl
%{_prefix}/bin/profile2mat.pl
%endif

# Note that we list the mandir specifically here, because we want all
# files found in that tree, because rpmbuild may have compressed them
# (e.g., foo.1.gz or foo.1.bz2) -- and we therefore don't know the
# exact filenames.
%files docs
%defattr(-, root, root, -)
%{_prefix}/man
%{_prefix}/share/doc
%if %{numversion} < 200
%dir %{_prefix}/share/vampirtrace
%{_prefix}/share/vampirtrace/doc
%endif
#%doc README INSTALL LICENSE

#############################################################################
#
# Changelog
#
#############################################################################
%changelog
* Tue Oct 28 2014 Josko Plazonic <plazonic@princeton.edu 
- add devtoolset based variant - entirely due to the need to have updated
  gfortran modules

* Wed Nov 21 2012 Josko Plazonic <plazonic@math.princeton.edu>
- upgrade to 1.6.3, add more macro configs configurable during the
  build, like intelminrelease

* Mon Sep 26 2011 Josko Plazonic <plazonic@math.princeton.edu>
- upgrade to 1.5.4

* Mon Mar 08 2010 Josko Plazonic <plazonic@math.princeton.edu>
- upgrade to 1.4.1

* Tue Dec 01 2009 Josko Plazonic <plazonic@math.princeton.edu>
- upgrade to 1.3.4

* Wed Aug 12 2009 Josko Plazonic <plazonic@math.princeton.edu>
- stop disabling memory managers - turns out now we need them for pinning
  which helps with IB performance

* Wed Jul 15 2009 Josko Plazonic <plazonic@math.princeton.edu>
- upgrade to 1.3.3

* Fri May 01 2009 Josko Plazonic <plazonic@math.princeton.edu>
- rebuild once more for new torque

* Thu Apr 23 2009 Josko Plazonic <plazonic@math.princeton.edu>
- upgrade to 1.3.2

* Wed Feb 18 2009 Josko Plazonic <plazonic@math.princeton.edu>
- add pathscale

* Tue Feb 17 2009 Josko Plazonic <plazonic@math.princeton.edu>
- upgrade to 1.3

* Fri Jan 09 2009 Josko Plazonic <plazonic@math.princeton.edu>
- tweak modules for 2cluster so that intel 9.1 is preferred over
  both pgi and intel-11.0

* Tue Dec 23 2008 Josko Plazonic <plazonic@math.princeton.edu>
- various fixes to allow all to build on rhel4

* Fri Dec 19 2008 Josko Plazonic <plazonic@math.princeton.edu>
- 1.2.8 plus gfortran fixes for suse

* Fri Feb 22 2008 Josko Plazonic <plazonic@math.princeton.edu>
- adapt to Suse 10.1 and try to add lustre support

* Mon Oct 01 2007 Josko Plazonic <plazonic@math.princeton.edu>
- rebuild but only for pgi to solve pthreads issues

* Fri Sep 15 2007 Josko Plazonic <plazonic@math.princeton.edu>
- lots of fixes plus special kludge to enable us to use openmpi/intel91
  and yet make it so that openmpi/intel resolves to the newest compiler
  compiled openmpi

* Thu Sep 14 2007 Josko Plazonic <plazonic@math.princeton.edu>
- drastically revise the build - now we use one generic spec file
  for all compilers with a define to build for a specific compiler
  NOTE: TAKE CARE TO KEEP ONE SPEC FILE!

* Mon Jun 25 2007 Josko Plazonic <plazonic@math.princeton.edu>
- 1.2.3 version

* Thu Jun 14 2007 Josko Plazonic <plazonic@math.princeton.edu>
- 1.2.2 version

* Wed Mar 21 2007 Josko Plazonic <plazonic@math.princeton.edu>
- 1.1.5 version

* Mon Feb 19 2007 Josko Plazonic <plazonic@math.princeton.edu>
- add support for pvfs2

* Fri Feb 2 2007 Josko Plazonic <plazonic@math.princeton.edu>
- 1.1.4 version

* Mon Jan 8 2007 Dennis McRitchie <dmcr@princeton.edu>
- Added command-line option "--with ib[=<dir>]" and "--without ib".
- Defaults to "with ib" on x86_64 systems, and to "without ib" on 32-bit systems.
- If "with ib", rpm and install directory name reflects that fact.
- Supports runtime, devel, and doc sub-packages.
- Use lib64 directory (instead of lib) on x86_64 systems.
- Install directories are nested by version, compiler, and arch.

* Mon Dec 18 2006 Dennis McRitchie <dmcr@princeton.edu>
- Uses modules to set up the Intel enviroment.

* Wed Dec 13 2006 Dennis McRitchie <dmcr@princeton.edu>
- Upgraded to openmpi v1.1.2
- No longer requires Intel compiler environment to be setup in advance.

* Fri Oct 06 2006 Dennis McRitchie <dmcr@princeton.edu>
- Changed it so additional configure options could be added with '--define "configure_options <configure-options>"', e.g., '--define "configure_options --without-memory-manager"'.
- Changed Intel compiler build code to only define the compilers (i.e., CC, CXX, F77, FC), and not the PATHs and FLAGS. So the assumption is that the Intel compiler environment would have been set up in advance of the build.
- Added --without-libnuma to the configure line.
- Commented out the two %doc lines in the files section that were causing duplicate file warnings.

* Fri Oct  6 2006 Jeff Squyes <jsquyres@cisco.com>
- Remove LANL section; they don't want it
- Add some help for OFED building
- Remove some outdated "rm -f" lines for executables that we no longer ship

* Wed Apr 26 2006 Jeff Squyres <jsquyres@cisco.com>
- Revamp files listings to ensure that rpm -e will remove directories
  if rpm -i created them.
- Simplify options for making modulefiles and profile.d scripts.
- Add oscar define.
- Ensure to remove the previous installation root during prep.
- Cleanup the modulefile specification and installation; also ensure
  that the profile.d scripts get installed if selected.
- Ensure to list sysconfdir in the files list if it's outside of the
  prefix.

* Wed Mar 30 2006 Jeff Squyres <jsquyres@cisco.com>
- Lots of bit rot updates
- Reorganize and rename the subpackages
- Add / formalize a variety of rpmbuild --define options
- Comment out the docs subpackage for the moment (until we have some
  documentation -- coming in v1.1!)

* Wed May 03 2005 Jeff Squyres <jsquyres@open-mpi.org>
- Added some defines for LANL defaults
- Added more defines for granulatirty of installation location for
  modulefile
- Differentiate between installing in /opt and whether we want to
  install environment script files
- Filled in files for man and mca-general subpackages

* Thu Apr 07 2005 Greg Kurtzer <GMKurtzer@lbl.gov>
- Added opt building
- Added profile.d/modulefile logic and creation
- Minor cleanups

* Fri Apr 01 2005 Greg Kurtzer <GMKurtzer@lbl.gov>
- Added comments
- Split package into subpackages
- Cleaned things up a bit
- Sold the code to Microsoft, and now I am retiring. Thanks guys!

* Wed Mar 23 2005 Mezzanine <mezzanine@kainx.org>
- Specfile auto-generated by Mezzanine

